/*
 * Copyright 2007 University of Konstanz
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * $Id: Activator.java 22 2007-04-25 09:17:58Z mkramis $
 * 
 */

package org.jscsi.whiskas;

import java.util.LinkedList;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.jscsi.whiskas.views.Control;
import org.jscsi.whiskas.views.VisualListener;
import org.osgi.framework.BundleContext;

/**
 * The activator class controls the plug-in life cycle.
 * @author Halldr Janetzko
 */
public class Activator extends AbstractUIPlugin {

  /** The plug-in ID.*/
  public static final String PLUGIN_ID = "ViSCSI";

  /** The shared instance.*/
  private static Activator plugin;

  /** Instance of Control Center.*/
  public Control ctr;

  /** List of histograms and patterns.*/
  public LinkedList < VisualListener > list_of_visualizer;

  /**
   * The constructor creates an instance of the
   * linked list.
   */
  public Activator() {

    plugin = this;
    list_of_visualizer = new LinkedList < VisualListener > ();
  }

  /**
   * Start of plugin.
   * @param context
   * @see org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext)
   */
  public void start(BundleContext context) throws Exception {

    super.start(context);
  }

  /**
   * End of plugin life circle.
   * @param context
   * @see org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext)
   */
  public void stop(BundleContext context) throws Exception {

    plugin = null;
    super.stop(context);
  }

  /**
   * Returns the shared instance.
   *
   * @return the shared instance
   */
  public static Activator getDefault() {
    return plugin;
  }

  /**
   * Returns an image descriptor for the image file at the given plug-in
   * relative path.
   *
   * @param path the path
   * @return the image descriptor
   */
  public static ImageDescriptor getImageDescriptor(String path) {

    return imageDescriptorFromPlugin(PLUGIN_ID, path);
  }
}
